/* CVector.hpp
 *
 *  Copyright (c) 2000-2006 by Ugo Chirico - http://www.ugosweb.com
 *  All Rights Reserved 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _VECTOR_HPP_
#define _VECTOR_HPP_

#ifndef DEFAULT_CAPACITY 
#define DEFAULT_CAPACITY 50
#define DEFAULT_LOADFACTOR 0.75
#endif

#define ERR_INDEX_OUT_OF_BOUND    0xC0001001L
#define ERR_VECTOR_IS_EMPTY		  0xC0001002L
#include <malloc.h>

template<class VALUE>
class UUCVector
{	

private:

	VALUE*	m_vector;
	unsigned int m_nCapacity;
    unsigned int m_nCount;
	unsigned int m_nIndex;		
    unsigned int m_threshold;
    float		 m_loadFactor;	

	void init()
	{
		m_vector = (VALUE*) malloc(sizeof(VALUE) * m_nCapacity);		
		m_threshold	 = (int)(m_nCapacity * m_nCapacity);		
		m_nCount	 = 0;
		m_nIndex	 = 0;

		for(unsigned int i = 0; i < m_nCapacity; i++)
		{
			m_vector[i] = NULL;
		}
	};
	
public:
	
	UUCVector(unsigned int initialCapacity, float loadFactor) 
	{
		m_loadFactor = loadFactor;		
		m_nCapacity  = initialCapacity;

		init();				
    };

    UUCVector(unsigned int initialCapacity) 
	{
		m_loadFactor = DEFAULT_LOADFACTOR;		
		m_nCapacity  = initialCapacity;
		init();								
	};
	
    UUCVector() 
	{
		m_loadFactor = DEFAULT_LOADFACTOR;
		m_nCapacity  = DEFAULT_CAPACITY;
		init();								
	};

    virtual ~UUCVector() 
	{
		removeAllElements();
		free(m_vector);
	};
	
	unsigned int size() const
	{
		return m_nCount;
    };
	
	unsigned int capacity() const
	{
		return m_nCapacity;
    };

    bool isEmpty() const
	{
		return m_nCount == 0;
    };

	void trimToSize() 
	{
		if (m_nCount < m_nCapacity) 
		{
			VALUE* oldVector = m_vector;
			m_vector = (VALUE*) malloc(sizeof(VALUE) * m_nCount);
			m_nCapacity = m_nCount;
			for(unsigned int i = 0; i < m_nCapacity; i++)
			{
				m_vector[i] = oldVector[i];
			}
			
			free(oldVector);
		}	    
    }

	void ensureCapacity(unsigned int minCapacity) 
	{
		unsigned int oldCapacity = m_nCapacity;

		if (minCapacity > m_nCapacity) 
		{			
			VALUE* oldVector = m_vector;
			m_vector = (VALUE*) malloc(sizeof(VALUE) * minCapacity);
			
			m_nCapacity = minCapacity;

			unsigned int i;
			for(i = 0; i < oldCapacity; i++)
			{
				m_vector[i] = oldVector[i];
			}				

			for(i = oldCapacity; i < m_nCapacity; i++)
			{
				m_vector[i] = NULL;
			}

			free(oldVector);			
		}
    }


	bool contains(VALUE elem) const
	{
		return indexOf(elem, 0) >= 0;
    }

	int indexOf(VALUE elem) const
	{
		return indexOf(elem, 0);
    }

	int indexOf(VALUE elem, unsigned int index) const
	{
		for (unsigned int i = index ; i < m_nCount; i++) 
		{
			if (elem == m_vector[i]) 
			{
				return i;
			}
		}

		return -1;
	}

	int lastIndexOf(VALUE elem) const
	{
		return lastIndexOf(elem, m_nCount - 1);
    }

	int lastIndexOf(VALUE elem, unsigned int index) const
	{
		for (int i = index ; i >= 0 ; i--) 
		{
			if (elem == m_vector[i]) 
			{
				return i;
			}
		}

		return -1;
	}

	VALUE elementAt(unsigned int index) const
	{
		if (index >= m_nCount || index < 0)		
			throw (long)ERR_INDEX_OUT_OF_BOUND;
		
		return m_vector[index];	
    }

	VALUE operator [] (unsigned int index) const
	{
		return elementAt(index);
	}

	VALUE firstElement() const
	{
		if (m_nCount == 0) 
		{
			throw (long)ERR_VECTOR_IS_EMPTY;
		}
	
		return m_vector[0];
    }

	VALUE lastElement() const
	{
		if (m_nCount == 0) 
		{
			throw (long)ERR_VECTOR_IS_EMPTY;
		}
	
		return m_vector[m_nCount - 1];
    }

	virtual void setElementAt(VALUE obj, unsigned int index)
	{
		if (index >= m_nCount) 
		{
			throw (long)ERR_INDEX_OUT_OF_BOUND;
		}

		m_vector[index] = obj;
    }

	virtual void removeElementAt(unsigned int index) 
	{
		if (index >= m_nCount) 
		{
			throw (long)ERR_INDEX_OUT_OF_BOUND;
		}
		else if (index < 0) 
		{
			throw (long)ERR_INDEX_OUT_OF_BOUND;
		}

		for(unsigned int i = index; i < m_nCount; i++)
		{
			m_vector[i] = m_vector[i + 1];
		}	
	
		m_vector[m_nCount] = NULL;

		m_nCount--;
    }

	virtual void insertElementAt(VALUE  obj, unsigned int index) 
	{
		unsigned int newcount = m_nCount + 1;
		if (index >= newcount) 
		{
			throw (long)ERR_INDEX_OUT_OF_BOUND;
		}

		if (newcount > m_nCapacity) 
		{
			ensureCapacity(newcount);
		}

		for(unsigned int i = m_nCount; i > index; i--)
		{
			m_vector[i] = m_vector[i - 1];
		}
		
		m_vector[index] = obj;

		m_nCount = newcount;
    }

	virtual void addElement(VALUE obj) 
	{
		insertElementAt(obj, m_nCount);	
    }

	virtual bool removeElement(VALUE obj) 
	{
		int i = indexOf(obj);
		if (i >= 0) 
		{
			removeElementAt(i);
			return true;
		}

		return false;
	}
	
	virtual void removeAllElements() 
	{
		while(m_nCount > 0) 
		{
			removeElementAt(m_nCount - 1);			
		}
		/*for (unsigned int i = 0; i < m_nCount;) 
		{
			removeElementAt(i);			
		}
		m_nCount = 0;*/
    }
};

#endif //_VECTOR_HPP_
